@@ -121,6 +121,7 @@ void drawdot(int xpos, int ypos)
       (*newarc)->style = FILLED | FILLSOLID | NOBORDER;
       calcarc(*newarc);
    }
+   incr_changes(objectdata);
 }
 
 /*----------------------*/
@@ -328,8 +329,10 @@ void labeltext(int keypressed, char *clientdata)
 
       Wprintf("");
 
-      if (hasstuff && (eventmode != TEXT3_MODE && eventmode != CATTEXT_MODE))
+      if (hasstuff && (eventmode != TEXT3_MODE && eventmode != CATTEXT_MODE)) {
 	 objectdata->parts++;
+	 incr_changes(objectdata);
+      }
       else if (!hasstuff && (eventmode == TEXT3_MODE)) objectdata->parts--;
 
       if ((!hasstuff) && (eventmode == CATTEXT_MODE)) {  /* can't have null labels! */ 
@@ -1508,6 +1511,7 @@ void join()
    /* clean up */
 
    objectdata->parts++;
+   incr_changes(objectdata);
    if (areastruct.selects > 0) free(areastruct.selectlist);
    areastruct.selects = 0;
    free(scount);
@@ -1569,6 +1573,7 @@ void wirebutton(XButtonEvent *event)
          XcSetFunction(GXcopy);
          XcSetForeground(newwire->color);
 	 objectdata->parts++;
+	 incr_changes(objectdata);
       }
       UDrawPolygon(areastruct.topobject, newwire);
       if (event->button == Button3)
