@@ -156,6 +156,40 @@ void endfiletrack(Widget w, caddr_t clientdata, XCrossingEvent *event)
    XSetFunction(dpy, areastruct.gc, areastruct.gctype);
 }
 
+/*----------------------------------------------------------------------*/
+/* Read a crash file to find the name of the original file.		*/
+/*----------------------------------------------------------------------*/
+
+char *getcrashfilename()
+{
+   FILE *fi;
+   char temp[256];
+   char *retstr = NULL, *tpos, *spos;
+   int slen;
+
+   if ((fi = fopen(_STR2, "r")) != NULL) {
+      while (fgets(temp, 255, fi) != NULL) {
+	 if ((tpos = strstr(temp, "Title:")) != NULL) {
+	    ridnewline(temp);
+	    tpos += 7;
+	    if ((spos = strrchr(temp, '/')) != NULL)
+	       tpos = spos + 1;
+	    retstr = (char *)malloc(1 + strlen(tpos));
+	    strcpy(retstr, tpos);
+	 }
+	 else if ((tpos = strstr(temp, "CreationDate:")) != NULL) {
+	    ridnewline(temp);
+	    tpos += 14;
+	    slen = strlen(retstr);
+	    retstr = (char *)realloc(retstr, 4 + slen + strlen(tpos));
+	    sprintf(retstr + slen, " (%s)", tpos);
+	    break;
+	 }
+      }
+   }
+   return retstr;
+}
+
 /*----------------------------------------------------------------------*/
 /* Crash recovery:  load the file, and link the tempfile name to it.    */
 /*----------------------------------------------------------------------*/
@@ -183,11 +217,11 @@ void findcrashfiles()
    uid_t userid = getuid();
    time_t recent = 0;
 
-   cwd = opendir("/tmp");
+   cwd = opendir(xobjs.tempdir);
    if (cwd == NULL) return;	/* No tmp directory, no tmp files! */
 
    while ((dp = readdir(cwd)) != NULL) {
-      sprintf(_STR, "/tmp/%s", dp->d_name);
+      sprintf(_STR, "%s/%s", xobjs.tempdir, dp->d_name);
       if (!strncmp(_STR + 5, "XC", 2)) {
          stat(_STR, &sbuf);
          if (sbuf.st_uid == userid) {
