@@ -224,8 +224,7 @@ void loadlgf(int mode)
 
 	       /* Get rid of newline character, if any */
 
-	       for (tstrp = lineptr; *tstrp != '\0' && *tstrp != '\n'; tstrp++);
-	       if (*tstrp == '\n') *tstrp = '\0';
+	       ridnewline(lineptr);
 
 	       /* forward to the label part of the input line */
 
