@@ -67,6 +67,9 @@ void printhelppix(Widget w)
 {
    static char *helptitle = "Macro Key Binding Summary:";
    static helpstruct helptext[] = {
+    { XCF_Start,	"Start"},
+    { XCF_Finish,	"Finish"},
+    { XCF_Cancel,	"Cancel"},
     { XCF_Zoom_In,	"Zoom in 3/2"},
     { XCF_Zoom_Out,	"Zoom out 3/2"},
     { XCF_Pan,		"Pan to center"},
