@@ -82,7 +82,8 @@ static char *function_names[NUM_FUNCTIONS] = {
    "Pop", "Push", "Delete", "Select", "Box", "Arc", "Text",
    "Exchange", "Copy", "Join", "Unjoin", "Spline", "Edit",
    "Undelete", "Select Save", "Unselect", "Dashed", "Dotted",
-   "Solid", "Prompt", "Dot", "Wire", "Nothing", "Exit"
+   "Solid", "Prompt", "Dot", "Wire", "Start", "Finish", "Cancel",
+   "Nothing", "Exit"
 
 #ifdef SCHEMA
    , "Netlist", "Swap", "Pin Label", "Info Label", "Connectivity",
@@ -319,20 +320,22 @@ char *key_to_string(int keywstate)
 
    if (ks != NoSymbol) str = XKeysymToString(ks);
 
+   kptr = (char *)malloc(32);
+   kptr[0] = '\0';
+   if (kmod & Mod1Mask) strcat(kptr, "Alt_");
+   if (kmod & ControlMask) strcat(kptr, "Control_");
+   if (kmod & LockMask) strcat(kptr, "Capslock_");
+   if (kmod & ShiftMask) strcat(kptr, "Shift_");
+
    if (str != NULL) {
-      kptr = (char *)malloc(strlen(str) + 32);
-      kptr[0] = '\0';
-      if (kmod & Mod1Mask) strcat(kptr, "Alt_");
-      if (kmod & ControlMask) strcat(kptr, "Control_");
-      if (kmod & LockMask) strcat(kptr, "Capslock_");
-      if (kmod & ShiftMask) strcat(kptr, "Shift_");
+      kptr = (char *)realloc(kptr, strlen(str) + 32);
       strcat(kptr, str);
    }
    else {
-      kptr = (char *)malloc(8);
-      if (kmod & Button1Mask) strcpy(kptr, "Button1");
-      else if (kmod & Button2Mask) strcpy(kptr, "Button2");
-      else if (kmod & Button3Mask) strcpy(kptr, "Button3");
+      kptr = (char *)realloc(kptr, 40);
+      if (kmod & Button1Mask) strcat(kptr, "Button1");
+      else if (kmod & Button2Mask) strcat(kptr, "Button2");
+      else if (kmod & Button3Mask) strcat(kptr, "Button3");
       else {
 	 kptr[0] = '0';
 	 kptr[1] = 'x';
@@ -435,7 +438,9 @@ int add_keybinding(const char *keystring, const char *fstring)
 }
 
 /*--------------------------------------------------------------*/
-/* Create list of default key bindings				*/
+/* Create list of default key bindings.				*/
+/* These are conditional upon any bindings set in the startup	*/
+/* file .xcircuitrc.						*/
 /*--------------------------------------------------------------*/
 
 void default_keybindings()
@@ -569,6 +574,10 @@ void default_keybindings()
    add_binding(XK_w, XCF_Wire);
    add_binding(BUTTON3, XCF_Nothing);
    add_binding(CTRL | ALT | XK_q, XCF_Exit);
+   add_binding(BUTTON1, XCF_Start);
+   add_binding(SHIFT | BUTTON1, XCF_Finish);
+   add_binding(BUTTON2, XCF_Finish);
+   add_binding(BUTTON3, XCF_Cancel);
 
 #ifdef SCHEMA
    add_binding(ALT | XK_q, XCF_Netlist);
