@@ -147,6 +147,28 @@ int pageposition(short libmode, XButtonEvent *event, int mode)
    }
 }
 
+/*------------------------------------------------------*/
+/* Find the number of other pages linked to the current */
+/* page (having the same filename).  Includes self, so 	*/
+/* result is the total number of pages in the output	*/
+/* file.						*/
+/*------------------------------------------------------*/
+
+short pagelinks()
+{
+   int i;
+   short count = 1;
+
+   for (i = 0; i < xobjs.pages; i++) {
+      if (i == areastruct.page) continue;
+      if (xobjs.pagelist[i]->pageobj == NULL) continue;
+      if (!strcmp(xobjs.pagelist[i]->filename,
+		  xobjs.pagelist[areastruct.page]->filename))
+	 count++;
+   }
+   return count;
+}
+
 /*------------------------------------------------------*/
 /* Test whether an object is a page, and return the	*/
 /* page number if it is.  Otherwise, return -1.		*/
@@ -326,7 +348,7 @@ void composepagelib(short mode)
    labelptr *pagelabel;
    stringpart *strptr;
    pointlist pointptr;
-   int bbuf, xdel, ydel, gxsize, gysize;
+   int margin, xdel, ydel, gxsize, gysize;
    int pages = (mode == PAGELIB) ? xobjs.pages : xobjs.numlibs;
    short fval = findhelvetica();
 
@@ -338,7 +360,7 @@ void composepagelib(short mode)
    libinst->parts = 0;
 
    computespacing(mode, &gxsize, &gysize, &xdel, &ydel);
-   bbuf = xdel / 40;
+   margin = xdel / 40;	/* margin between pages */
 
    for (i = 0; i < pages; i++) {
       libobj = (mode == PAGELIB) ? xobjs.pagelist[i]->pageobj
@@ -357,17 +379,17 @@ void composepagelib(short mode)
       (*drawbox)->number = 4;
       (*drawbox)->points = (pointlist) malloc(4 * sizeof(XPoint));
       pointptr = (*drawbox)->points;
-      pointptr->x = (i % gxsize) * xdel + bbuf;
-      pointptr->y = -(i / gxsize) * ydel - bbuf;
+      pointptr->x = (i % gxsize) * xdel + margin;
+      pointptr->y = -(i / gxsize) * ydel - margin;
       pointptr = (*drawbox)->points + 1;
-      pointptr->x = ((i % gxsize) + 1) * xdel - bbuf;
-      pointptr->y = -(i / gxsize) * ydel - bbuf;
+      pointptr->x = ((i % gxsize) + 1) * xdel - margin;
+      pointptr->y = -(i / gxsize) * ydel - margin;
       pointptr = (*drawbox)->points + 2;
-      pointptr->x = ((i % gxsize) + 1) * xdel - bbuf;
-      pointptr->y = -((i / gxsize) + 1) * ydel + bbuf;
+      pointptr->x = ((i % gxsize) + 1) * xdel - margin;
+      pointptr->y = -((i / gxsize) + 1) * ydel + margin;
       pointptr = (*drawbox)->points + 3;
-      pointptr->x = (i % gxsize) * xdel + bbuf;
-      pointptr->y = -((i / gxsize) + 1) * ydel + bbuf;
+      pointptr->x = (i % gxsize) * xdel + margin;
+      pointptr->y = -((i / gxsize) + 1) * ydel + margin;
       libinst->parts++;
 
       /* each page gets its name printed at the bottom */
@@ -375,7 +397,7 @@ void composepagelib(short mode)
       if (libobj != NULL) {
 	 NEW_LABEL(pagelabel, libinst);
 	 labeldefaults(*pagelabel, False, (pointptr->x + (pointptr-1)->x) / 2,
-		pointptr->y - 5);
+			pointptr->y - 5);
 	 (*pagelabel)->color = DEFAULTCOLOR;
 	 (*pagelabel)->scale = 0.75;
 	 (*pagelabel)->string->data.font = fval;
@@ -602,9 +624,6 @@ void composelib(short mode)
 	 strptr->data.string = (char *) malloc(1 + strlen((*libobj)->name));
          strcpy(strptr->data.string, (*libobj)->name);
          (*drawname)->justify = TOP | NOTBOTTOM | NOTLEFT;
-#ifdef SCHEMA
-         (*drawname)->pin = False;
-#endif
 
          if ((*libobj)->_width > 170)
             (*drawname)->position.x = xpos + ((*libobj)->_width >> 1);
