@@ -906,37 +906,24 @@ void setfilename(Widget w, char **dataptr)
 {
    short cpage, depend = 0;
    objectptr checkpage;
+   char *oldstr = xobjs.pagelist[areastruct.page]->filename;
 
    if (!strcmp(*dataptr, _STR2)) return;   /* no change in string */
 
-   /* Check for dependencies on this string from other pages.        */
-   /* If there are no other pointers to this string, we can free it. */
-
-   for (cpage = 0; cpage < xobjs.pages; cpage++) {
-      checkpage = xobjs.pagelist[cpage]->pageobj;
-      if ((checkpage != NULL) && (cpage != areastruct.page))  
-	 if (xobjs.pagelist[cpage]->filename == xobjs.pagelist[areastruct.page]->filename)  
-	    depend = 1;
-   }
-
-   /* check if the new filename is an existing filename */
+   /* Make the change to the current page */
+   xobjs.pagelist[areastruct.page]->filename = strdup(_STR2);
 
+   /* All existing filenames which match the old string should also be changed */
    for (cpage = 0; cpage < xobjs.pages; cpage++) {
       checkpage = xobjs.pagelist[cpage]->pageobj;
       if ((checkpage != NULL) && (cpage != areastruct.page)) {
-	 if (!strcmp(_STR2, xobjs.pagelist[cpage]->filename)) {
-	    if (!depend) free(xobjs.pagelist[areastruct.page]->filename);
-	    xobjs.pagelist[areastruct.page]->filename = xobjs.pagelist[cpage]->filename;
-	    break;
+	 if (!strcmp(xobjs.pagelist[cpage]->filename, oldstr)) {
+	    free(xobjs.pagelist[cpage]->filename);
+	    xobjs.pagelist[cpage]->filename = strdup(_STR2);
 	 }
       }
    }
-   if (cpage == xobjs.pages) {
-      if (!depend) free(xobjs.pagelist[areastruct.page]->filename);
-      xobjs.pagelist[areastruct.page]->filename = (char *) malloc((strlen(_STR2)
-		+ 1) * sizeof(char));
-      strcpy(xobjs.pagelist[areastruct.page]->filename, _STR2);
-   }
+   free(oldstr);
 }
 
 /*----------------------------------------------------------------*/
