@@ -64,6 +64,10 @@ extern void endvbar(Widget, caddr_t, XButtonEvent *);
 extern void zoombox(Widget, caddr_t, caddr_t);
 extern void zoomin(Widget, caddr_t, XButtonEvent *);
 extern void zoomout(Widget, caddr_t, XButtonEvent *);
+extern void panbutton(u_int, XButtonEvent *);
+extern void zoominrefresh(Widget, caddr_t, XButtonEvent *);
+extern void zoomoutrefresh(Widget, caddr_t, XButtonEvent *);
+extern void panrefresh(u_int, XButtonEvent *);
 extern void centerpan(Widget, caddr_t, caddr_t);
 extern void checkwarp(XPoint *);
 extern int checkcycle(short, short);
@@ -71,7 +75,7 @@ extern void nextpolycycle(polyptr, short);
 extern void nextsplinecycle(splineptr, short);
 extern void nextarccycle(arcptr, short);
 extern short getkeynum();
-extern void keyrelease(Widget, caddr_t, XKeyEvent *);
+extern void buttonhandler(Widget, caddr_t, XButtonEvent *);
 extern void keyhandler(Widget, caddr_t, XKeyEvent *);
 extern void setsnap(short);
 extern void snapobject();
@@ -93,7 +97,6 @@ extern void elhflip(genericptr *);
 extern void elvflip(genericptr *);
 extern void objectflip();
 extern void objectvflip();
-extern void panbutton(u_int, XButtonEvent *);
 extern void makepress(XtIntervalId, caddr_t);
 extern void startrotate(Widget, caddr_t, caddr_t);
 extern void startcopy(Widget, caddr_t, caddr_t);
@@ -116,8 +119,8 @@ extern void arccopy(arcptr, arcptr);
 extern void polycopy(polyptr, polyptr);
 extern void splinecopy(splineptr, splineptr);
 extern void copybutton(XButtonEvent *);
-extern void selectbutton(Widget, caddr_t, XButtonEvent *);
-extern void releasebutton(Widget, caddr_t, XButtonEvent *);
+extern void selectbutton(XButtonEvent *);
+extern void releasebutton(XButtonEvent *);
 extern void pathbutton(genericptr *, XButtonEvent *);
 extern void resizearea(Widget, caddr_t, caddr_t);
 extern void drawarea(Widget, caddr_t, caddr_t);
@@ -125,11 +128,12 @@ extern void drawarea(Widget, caddr_t, caddr_t);
 /* from filelist.c: */
 
 extern int fcompare(const void *, const void *);
-extern void crashrecover();
-extern void findcrashfiles();
 extern void dragfilebox(Widget, caddr_t, XMotionEvent *);
 extern void startfiletrack(Widget, caddr_t, XCrossingEvent *);
 extern void endfiletrack(Widget, caddr_t, XCrossingEvent *);
+extern char *getcrashfilename();
+extern void crashrecover();
+extern void findcrashfiles();
 extern void listfiles(Widget, caddr_t, caddr_t);
 extern void newfilelist(Widget, Widget);
 extern void fileselect(Widget, Widget, XButtonEvent *);
@@ -140,6 +144,7 @@ extern int lookdirectory(char *);
 
 /* from files.c: */
 
+extern void ridnewline(char *);
 extern void free_single(genericptr *);
 extern void reset(objectptr, short);
 extern void topreset();
@@ -303,6 +308,7 @@ extern void remove_keybinding(const char *, const char *);
 extern short findhelvetica();
 extern void catreturn();
 extern int pageposition(short, XButtonEvent *, int);
+extern short pagelinks();
 extern int is_page(objectptr);
 extern int is_library(objectptr);
 extern void pagecatbutton(XButtonEvent *);
@@ -430,6 +436,7 @@ extern Boolean neardist(long);
 extern Boolean nearpoint(XPoint *, XPoint *);
 extern int make_tmp_pin(objectptr, XPoint *);
 extern Boolean searchconnect(XPoint *, int, objinstptr);
+extern int translateup(int, objectptr, objectptr, objinstptr);
 extern void addpoly(objectptr, polyptr, int);
 extern long zsign(long, long);
 extern void mergecleanup(objectptr);
@@ -456,7 +463,7 @@ extern void outputpcb(objectptr, FILE *);
 extern void freepcb();
 extern void freenets(objectptr);
 extern void freeglobals();
-extern void cleartraversed(objectptr);
+extern int cleartraversed(objectptr, int);
 extern void clearpins(objectptr);
 #endif
 
@@ -495,6 +502,11 @@ extern void init_interpreter();
 extern void exit_interpreter();
 #endif
 
+#ifdef HAVE_XPM
+extern int pytoolbuttons();
+extern Widget pytoolbutton(int);
+#endif
+
 /* from rcfile.c: */
 
 extern short execcommand(short, char *);
@@ -537,7 +549,7 @@ extern void pinconvert(labelptr, u_int);
 extern void dopintype(Widget, u_int, caddr_t);
 extern void setsymschem();
 extern int findpageobj(objectptr);
-extern void findsubschems(int, objectptr);
+extern int findsubschems(int, objectptr, int);
 extern void copypinlabel(labelptr);
 extern int checkschem(objectptr, char *);
 extern int checksym(objectptr, char *);
@@ -605,9 +617,11 @@ extern void fontcatbutton(XButtonEvent *);
 
 /* from xcircuit.c: */
 
+extern void printeventmode();
 extern void Wprintf(char *);
 extern void popupprompt(Widget, char *, char *, void (*function)(),
                         buttonsave *, Boolean);
+extern void getproptext(Widget, propstruct *, caddr_t);
 extern int rgb_alloccolor(int, int, int);
 extern void addtocolorlist(Widget, int);
 extern void addnewcolorentry(int);
@@ -617,8 +631,11 @@ extern void W1printf(char *);
 extern void W2printf(char *);
 extern void destroypopup(Widget, popupstruct *, caddr_t);
 extern int xc_alloccolor(char *);
+extern void dointr(int);
 extern void DoNothing(Widget, caddr_t, caddr_t);
-extern void quit(Widget, caddr_t, caddr_t);
+extern u_short countchanges(char **);
+extern void quitcheck(Widget, caddr_t, caddr_t);
+extern void quit(Widget, caddr_t);
 extern void resizetoolbar();
 extern void updatetext(Widget, WidgetList, caddr_t);
 extern void makecursors();
