@@ -30,6 +30,37 @@
 #include "Xw/Xw.h"
 #include "Xw/MenuBtn.h"
       
+#ifdef HAVE_XPM
+#include <X11/xpm.h>
+/* "unknown" tool */
+static char *q_xpm[] = {
+"20 20 4 1",
+".      c #FFFFFFFFFFFF",
+"X      c #C2C2CECEFFFF",
+"o      c #79799090FFFF",
+"O      c #00000000FFFF",
+"....................",
+".......XXXX.........",
+".....XoOOOOoX.......",
+".....oOOoXOOOX......",
+"....oOOX..XOOo......",
+"...XOOX....XOO......",
+"...XOO.....XOO......",
+"....XX.....oOo......",
+"..........XOO.......",
+".........XOO........",
+"........XOOX........",
+".......XOO..........",
+".......OOX..........",
+".......oOOXo........",
+".......XOOO.........",
+"........XX..........",
+".......XOOX.........",
+".......oOOo.........",
+".......XOOX.........",
+"...................."};
+#endif
+
 /*----------------------------------------------------------------------*/
 /* Local includes                                                       */
 /*----------------------------------------------------------------------*/
@@ -53,12 +84,20 @@ static PyObject *xc_pause(PyObject *, PyObject *);
 static PyObject *xc_refresh(PyObject *, PyObject *);
 static PyObject *xc_bind(PyObject *, PyObject *);
 static PyObject *xc_unbind(PyObject *, PyObject *);
+static PyObject *xc_simplepopup(PyObject *, PyObject *);
 static PyObject *xc_popupprompt(PyObject *, PyObject *);
 static PyObject *xc_filepopup(PyObject *, PyObject *);
 static PyObject *xc_newbutton(PyObject *, PyObject *);
 static PyObject *xc_page(PyObject *, PyObject *);
+static PyObject *xc_pan(PyObject *, PyObject *);
+static PyObject *xc_zoom(PyObject *, PyObject *);
 static PyObject *xc_getcursor();
+static PyObject *xc_getwindow();
 static PyObject *xc_reset();
+static PyObject *xc_netlist();
+#ifdef HAVE_XPM
+static PyObject *xc_newtool(PyObject *, PyObject *);
+#endif
 
 int string_to_type(const char *);
 
@@ -78,6 +117,9 @@ extern Window win;
 extern keybinding *keylist;
 extern colorindex *colorlist;
 extern ApplicationData appdata;
+#ifdef HAVE_XPM
+extern Widget toolbar;
+#endif
 
 /*----------------------------------------------------------------------*/
 
@@ -98,12 +140,17 @@ short flags;
 
 /*--------------------------------------------------------------*/
 /* Extract text from a popup and call the indicated python	*/
-/* function.							*/
+/* function.   If there is no text, function is called without	*/
+/* arguments.							*/
 /*--------------------------------------------------------------*/
 
 void pypromptvalue(Widget w, char *functionptr)
 {
-   sprintf(_STR, "%s('%s')\n", functionptr, _STR2);
+   if (strlen(_STR2) == 0)
+      sprintf(_STR, "%s()\n", functionptr);
+   else
+      sprintf(_STR, "%s('%s')\n", functionptr, _STR2);
+
    PyRun_SimpleString(_STR);
    refresh(NULL, NULL, NULL);
 }
@@ -113,7 +160,8 @@ void pypromptvalue(Widget w, char *functionptr)
 /* python function.						*/
 /*--------------------------------------------------------------*/
 
-static PyObject *pypopupprompt(PyObject *self, PyObject *args, Boolean dofile)
+static PyObject *pypopupprompt(PyObject *self, PyObject *args, Boolean dofile,
+	char *displaystr)
 {
    const char *pfunction = NULL;
    const char *prompt = NULL;
@@ -122,13 +170,9 @@ static PyObject *pypopupprompt(PyObject *self, PyObject *args, Boolean dofile)
    if (!PyArg_ParseTuple(args, "ss:popupprompt", &prompt, &pfunction))
       return PyInt_FromLong(-1L);
 
-   _STR2[0] = '\0';
 
-   savebutton->button = NULL;
-   savebutton->buttoncall = NULL;
-   savebutton->dataptr = (char *)pfunction;
-
-   popupprompt(areastruct.area, (char *)prompt, _STR2,
+   getgeneric(savebutton, NULL, NULL, (void *)pfunction);
+   popupprompt(areastruct.area, (char *)prompt, displaystr,
 		pypromptvalue, savebutton, dofile);
 
    return PyInt_FromLong(0L);
@@ -140,12 +184,18 @@ static PyObject *pypopupprompt(PyObject *self, PyObject *args, Boolean dofile)
 
 static PyObject *xc_popupprompt(PyObject *self, PyObject *args)
 {
-   return pypopupprompt(self, args, False);
+   return pypopupprompt(self, args, False, "");
 }
 
 static PyObject *xc_filepopup(PyObject *self, PyObject *args)
 {
-   return pypopupprompt(self, args, True);
+   return pypopupprompt(self, args, True, "");
+}
+
+static PyObject *xc_simplepopup(PyObject *self, PyObject *args)
+{
+   _STR2[0] = '\0';
+   return pypopupprompt(self, args, False, NULL);
 }
 
 /*--------------------------------------------------------------*/
@@ -206,6 +256,7 @@ static PyObject *xc_new(PyObject *self, PyObject *args)
          return NULL;
    } 
    objectdata->parts++;
+   incr_changes(objectdata);
    return PyInt_FromLong((long)(*newgen));
 }
 
@@ -388,7 +439,7 @@ stringpart *PySetStringParts(PyObject *dval)
 /* Convert a stringpart* to a Python list 			*/
 /*--------------------------------------------------------------*/
 
-static PyObject *PyGetStringParts(stringpart *thisstring)
+PyObject *PyGetStringParts(stringpart *thisstring)
 {
    PyObject *lstr, *sdict, *stup;
    int i, llen;
@@ -684,7 +735,7 @@ static PyObject *xc_getlibrary(PyObject *self, PyObject *args)
 static PyObject *xc_getobject(PyObject *self, PyObject *args)
 {
    objectptr thisobj = NULL, *libobj;
-   PyObject *rdict, *dlist;
+   PyObject *rdict, *dlist, *tpos;
    const char *oname;
    int i, j, k;
 
@@ -702,7 +753,14 @@ static PyObject *xc_getobject(PyObject *self, PyObject *args)
       if (thisobj != NULL) break;
    }
 
-   if (thisobj==NULL) return NULL;
+   if (thisobj == NULL) { 	/* try the page objects */
+      for (k = 0; k < xobjs.pages; k++) {
+         if (!(thisobj = xobjs.pagelist[k]->pageobj)) continue;
+	 if (!strcmp(oname, thisobj->name))
+	    break;
+      }
+      if (k == xobjs.pages) return NULL;	/* not found */
+   }
 
    /* return all the object's properties as a dictionary */
 
@@ -714,6 +772,18 @@ static PyObject *xc_getobject(PyObject *self, PyObject *args)
 	PyInt_FromLong((long)(thisobj->width)));
    PyDict_SetItem(rdict, PyString_FromString("height"), 
 	PyInt_FromLong((long)(thisobj->height)));
+   PyDict_SetItem(rdict, PyString_FromString("viewscale"), 
+	PyFloat_FromDouble((double)(thisobj->viewscale)));
+
+   tpos = PyTuple_New(2);
+   PyTuple_SetItem(tpos, 0, PyInt_FromLong((long)thisobj->lowerleft.x));
+   PyTuple_SetItem(tpos, 1, PyInt_FromLong((long)thisobj->lowerleft.y));
+   PyDict_SetItem(rdict, PyString_FromString("boundingbox"), tpos);
+
+   tpos = PyTuple_New(2);
+   PyTuple_SetItem(tpos, 0, PyInt_FromLong((long)thisobj->pcorner.x));
+   PyTuple_SetItem(tpos, 1, PyInt_FromLong((long)thisobj->pcorner.y));
+   PyDict_SetItem(rdict, PyString_FromString("viewcorner"), tpos);
 
    dlist = PyList_New(thisobj->parts);
    for (i = 0; i < thisobj->parts; i++)
@@ -745,6 +815,35 @@ static PyObject *xc_reset()
    return PyInt_FromLong(0L);
 }
 
+#ifdef SCHEMA
+/*--------------------------------------------------------------*/
+/* Get a netlist						*/
+/*--------------------------------------------------------------*/
+
+extern u_int subindex;  /* count for labeling spice subcircuits */
+extern PyObject *pyglobals(objectptr);
+extern PyObject *pytoplevel(objectptr);
+
+static PyObject *xc_netlist()
+{
+   PyObject *rdict = NULL;
+
+   if (cleartraversed(objectdata, 0) == -1)
+      PyErr_SetString(PyExc_TypeError, "Error:  Check circuit for infinite recursion.");
+   else {
+      createnets(False, "spice");
+      subindex = 1;
+   
+      rdict = PyDict_New();
+      PyDict_SetItem(rdict, PyString_FromString("globals"), pyglobals(objectdata));
+      PyDict_SetItem(rdict, PyString_FromString("circuit"), pytoplevel(objectdata));
+
+      destroynets();
+   }
+   return rdict;
+}
+#endif
+
 /*--------------------------------------------------------------*/
 /* Go to the specified page					*/
 /*--------------------------------------------------------------*/
@@ -754,12 +853,98 @@ static PyObject *xc_page(PyObject *self, PyObject *args)
    int pageno;
 
    if (!PyArg_ParseTuple(args, "i:page", &pageno))
-      return NULL;
+      return PyInt_FromLong((long)areastruct.page);
 
    newpage(pageno - 1);
    return PyInt_FromLong((long)pageno);
 }
 
+/*--------------------------------------------------------------*/
+/* Zoom								*/
+/*--------------------------------------------------------------*/
+
+static PyObject *xc_zoom(PyObject *self, PyObject *args)
+{
+   float factor, save;
+
+   if (!PyArg_ParseTuple(args, "f:zoom", &factor))
+      return NULL;
+
+   if (factor <= 0) {
+      PyErr_SetString(PyExc_TypeError, "Argument must be positive.");
+      return NULL;
+   }
+   
+   save = areastruct.zoomfactor;
+
+   if (factor < 1.0) {
+      areastruct.zoomfactor = 1.0 / factor;
+      zoomout(NULL, Number(0), NULL);
+   }
+   else {
+      areastruct.zoomfactor = factor;
+      zoomin(NULL, Number(0), NULL);
+   }
+
+   areastruct.zoomfactor = save;
+   return PyFloat_FromDouble((double)objectdata->viewscale);
+}
+
+/*--------------------------------------------------------------*/
+/* Pan								*/
+/*--------------------------------------------------------------*/
+
+static PyObject *xc_pan(PyObject *self, PyObject *args)
+{
+   PyObject *cornerpos, *xobj, *yobj = NULL, *pval;
+   int x, y;
+   XButtonEvent bevent;
+   XPoint upoint, wpoint;
+
+   if (!PyArg_ParseTuple(args, "O|O:pan", &xobj, &yobj))
+      return NULL;
+
+   if (yobj == NULL) {
+      if (PyTuple_Check(xobj) && PyTuple_Size(xobj) == 2) {
+         if ((pval = PyTuple_GetItem(xobj, 0)) != NULL)
+            upoint.x = PyInt_AsLong(pval);
+         if ((pval = PyTuple_GetItem(xobj, 1)) != NULL)
+            upoint.y = PyInt_AsLong(pval);
+      }
+      else return NULL;
+   }
+   else {
+      upoint.x = PyInt_AsLong(xobj);
+      upoint.y = PyInt_AsLong(yobj);
+   }
+
+   user_to_window(upoint, &wpoint);
+   bevent.x = wpoint.x;
+   bevent.y = wpoint.y;
+   panbutton((u_int)5, &bevent);
+
+   cornerpos = PyTuple_New(2);
+   PyTuple_SetItem(cornerpos, 0, PyInt_FromLong((long)areastruct.pcorner->x));
+   PyTuple_SetItem(cornerpos, 1, PyInt_FromLong((long)areastruct.pcorner->y));
+
+   return cornerpos;
+}
+
+/*--------------------------------------------------------------*/
+/* Get the window size						*/
+/*--------------------------------------------------------------*/
+
+static PyObject *xc_getwindow()
+{
+   PyObject *windowsize;
+
+   windowsize = PyTuple_New(2);
+   PyTuple_SetItem(windowsize, 0, PyInt_FromLong((long)areastruct.width));
+   PyTuple_SetItem(windowsize, 1, PyInt_FromLong((long)areastruct.height));
+
+   return windowsize;
+}
+
 /*--------------------------------------------------------------*/
 /* Get the cursor position					*/
 /*--------------------------------------------------------------*/
@@ -889,9 +1074,11 @@ static PyObject *xc_getattr(PyObject *self, PyObject *args)
 	 PyDict_SetItem(rdict, PyString_FromString("position"), 
 		make_pair(&(TOOBJINST(gelem)->position)));
 	 PyDict_SetItem(rdict, PyString_FromString("rotation"),
-		PyInt_FromLong((long)TOLABEL(gelem)->rotation));
+		PyInt_FromLong((long)TOOBJINST(gelem)->rotation));
 	 PyDict_SetItem(rdict, PyString_FromString("scale"),
-		PyFloat_FromDouble((double)TOLABEL(gelem)->scale));
+		PyFloat_FromDouble((double)TOOBJINST(gelem)->scale));
+	 PyDict_SetItem(rdict, PyString_FromString("name"),
+		PyString_FromString(TOOBJINST(gelem)->thisobject->name));
 	 break;
    }
    return rdict;
@@ -1175,7 +1362,16 @@ static PyObject *xc_set(PyObject *self, PyObject *args)
       if (i == fontcount)
 	 loadfontfile((char *)sarg2);
 
-      areastruct.psfont = i;
+      /* loadfontfile() may load multiple fonts, so we have to check again */
+      /* to see which one matches.					   */
+
+      for (i = 0; i < fontcount; i++) {
+	 if (!strcmp(fonts[i].psname, sarg2)) {
+            areastruct.psfont = i;
+            setdefaultfontmarks();
+	    break;
+	 }
+      }
    }
    else if (!strcmp(sarg1, "fontscale")) {
       sscanf(sarg2, "%f", &areastruct.textscale);
@@ -1340,6 +1536,14 @@ static PyObject *xc_bind(PyObject *self, PyObject *args)
    if (!PyArg_ParseTuple(args, "|ss:bind", &keyname, &function))
       return NULL;
 
+   /* If we're in .xcircuitrc and we're not overriding the key	    */
+   /* bindings, then the first binding function should precipitate  */
+   /* calling the defaults, then specify the bindings as overridden */
+   if (!(flags & KEYOVERRIDE)) {
+      default_keybindings();
+      flags |= KEYOVERRIDE;
+   }
+
    /* No arguments?  Return a dictionary of bound pairs */
    if (!keyname) {
       PyObject *key, *value;
@@ -1398,6 +1602,14 @@ static PyObject *xc_unbind(PyObject *self, PyObject *args)
    if (!PyArg_ParseTuple(args, "ss:unbind", &keyname, &function))
       return NULL;
 
+   /* If we're in .xcircuitrc and we're not overriding the key	    */
+   /* bindings, then the first binding function should precipitate  */
+   /* calling the defaults, then specify the bindings as overridden */
+   if (!(flags & KEYOVERRIDE)) {
+      default_keybindings();
+      flags |= KEYOVERRIDE;
+   }
+
    remove_keybinding(keyname, function);
    return PyString_FromString(keyname);
 }
@@ -1450,6 +1662,20 @@ int pybutton(Widget w, caddr_t clientdata, caddr_t calldata)
    return status;
 }
 
+/*--------------------------------------------------------------*/
+/* Callback procedure for Python-generated toolbar tool		*/
+/*--------------------------------------------------------------*/
+
+int pytool(Widget w, caddr_t clientdata, caddr_t calldata)
+{  
+   int status;
+
+   sprintf(_STR2, "tooldict[%d]()\n", w);
+   status = PyRun_SimpleString(_STR2);
+   refresh(NULL, NULL, NULL);
+   return status;
+}
+
 /*--------------------------------------------------------------*/
 /* Add a new button to the specified (by name) cascade menu	*/
 /*--------------------------------------------------------------*/
@@ -1489,6 +1715,78 @@ static PyObject *xc_newbutton(PyObject *self, PyObject *args)
    return PyInt_FromLong((long)status);
 }
 
+#ifdef HAVE_XPM
+
+/*--------------------------------------------------------------*/
+/* Add a new tool to the toolbar				*/
+/*--------------------------------------------------------------*/
+
+static PyObject *xc_newtool(PyObject *self, PyObject *args)
+{
+   const char *thint = NULL;
+   const char *pixfile = NULL;
+   const char *newtname = NULL;
+   const char *pfunction = NULL;
+   int status;
+   Arg	wargs[8];
+   int  i, n = 0;
+   Widget newtool, cascade, tbutton;
+   XImage *iret;
+   XpmAttributes attr;
+
+   attr.valuemask = XpmSize | XpmCloseness;
+   attr.closeness = 65536;
+
+   if (!PyArg_ParseTuple(args, "sss|s:newtool", &newtname, &pfunction, &pixfile,
+		&thint))
+      return NULL;
+
+   if (XpmReadFileToImage(dpy, (char *)pixfile, &iret, NULL, &attr) < 0) {
+      sprintf(_STR, "Cannot find or open pixmap file \'%s\'", pixfile);
+      Wprintf(_STR);
+      XpmCreateImageFromData(dpy, q_xpm, &iret, NULL, &attr);
+   }
+
+   XtnSetArg(XtNlabelType, XwIMAGE);
+   XtnSetArg(XtNlabelImage, iret);
+   XtnSetArg(XtNwidth, attr.width + 4);
+   XtnSetArg(XtNheight, attr.height + 4);
+   XtnSetArg(XtNborderWidth, TBBORDER);
+   XtnSetArg(XtNnoPad, True);
+   if (thint != NULL) {
+      XtnSetArg(XtNhint, thint);
+      XtnSetArg(XtNhintProc, Wprintf);
+   }
+   newtool = XtCreateWidget(newtname, XwmenubuttonWidgetClass, toolbar, wargs, n);
+
+   XtAddCallback (newtool, XtNselect, (XtCallbackProc)pybutton, newtool);
+   XtManageChild(newtool);
+
+   sprintf(_STR2, "tooldict[%d] = %s\n", newtool, pfunction);
+   status = PyRun_SimpleString(_STR2);
+   return PyInt_FromLong((long)status);
+}
+
+/*--------------------------------------------------------------*/
+/* Return the size of tooldict					*/
+/*--------------------------------------------------------------*/
+ 
+int pytoolbuttons()
+{
+   return 0;
+}
+
+/*--------------------------------------------------------------*/
+/* Return the widget address of the Nth entry in tooldict	*/
+/*--------------------------------------------------------------*/
+
+Widget pytoolbutton(int index)
+{
+   return NULL;
+}
+
+#endif
+
 /*----------------------------------------------------------------*/
 /* Execute a python command from the key-function pair dictionary */
 /*----------------------------------------------------------------*/
@@ -1523,11 +1821,21 @@ static PyMethodDef xc_methods[] = {
    {"bind",		xc_bind,	1},
    {"unbind",		xc_unbind,	1},
    {"getcursor",	xc_getcursor,	1},
+   {"getwindow",	xc_getwindow,	1},
+   {"zoom",		xc_zoom,	1},
+   {"pan",		xc_pan,		1},
    {"popupprompt",	xc_popupprompt, 1},
    {"filepopup",	xc_filepopup,	1},
+   {"simplepopup",	xc_simplepopup,	1},
    {"newbutton",	xc_newbutton,	1},
    {"reset",		xc_reset,	1},
    {"page",		xc_page,	1},
+#ifdef HAVE_XPM
+   {"newtool",		xc_newtool,	1},
+#endif
+#ifdef SCHEMA
+   {"netlist",		xc_netlist,	1},
+#endif
    {NULL,	NULL}			/* sentinel */
 };
 
@@ -1543,6 +1851,9 @@ void init_interpreter()
    PyRun_SimpleString("from xc import *\n");
    PyRun_SimpleString("keydict = {}\n");	/* initialize key/function pairs */
    PyRun_SimpleString("buttondict = {}\n");  /* initialize button/function pairs */
+   PyRun_SimpleString("tooldict = {}\n");  /* initialize tool/function pairs */
+   sprintf(_STR, "xc_version = %2.1f\n", PROG_VERSION);
+   PyRun_SimpleString(_STR);
 }
 
 /*--------------------------------------------------------------*/
@@ -1640,9 +1951,16 @@ void loadrcfile()
 
    if (!(flags & FONTOVERRIDE)) {
       loadfontfile("Helvetica");
-      for (i = 0; i < fontcount; i++)
-	 if (!strcmp(fonts[i].psname, "Helvetica")) areastruct.psfont = i;
+      if (areastruct.psfont == -1)
+         for (i = 0; i < fontcount; i++)
+	    if (!strcmp(fonts[i].psname, "Helvetica")) {
+	       areastruct.psfont = i;
+	       break;
+	    }
    }
+
+   if (areastruct.psfont == -1) areastruct.psfont = 0;
+
    setdefaultfontmarks();
       
    /* arrange the loaded libraries */
