@@ -193,6 +193,11 @@ short execcommand(short pflags, char *cmdptr)
    else if (!strcmp(cmdptr, "bind")) {
       int args;
       char *sptr;
+
+      if (!(flags & KEYOVERRIDE)) {
+	 default_keybindings();
+         flags |= KEYOVERRIDE;
+      }
       if ((args = sscanf(argptr, "%149s", _STR)) == 1) {
 	 char *binding;
 	 int func = -1, keywstate;
@@ -220,6 +225,10 @@ short execcommand(short pflags, char *cmdptr)
    /* destroy macro */
 
    else if (!strcmp(cmdptr, "unbind")) {
+      if (!(flags & KEYOVERRIDE)) {
+	 default_keybindings();
+         flags |= KEYOVERRIDE;
+      }
       sscanf(argptr, "%149s %249s", _STR, _STR2);
       remove_keybinding(_STR, _STR2);
    }
@@ -277,6 +286,7 @@ short execcommand(short pflags, char *cmdptr)
 	       loadfontfile(value);
 	    }
    	    areastruct.psfont = i;
+	    setdefaultfontmarks();
 	 }
       }
       else if (!strncmp(argptr, "boxedit", 7)) {
