@@ -147,12 +147,27 @@ void wait_for_page()
    for(;;) {
       XNextEvent(dpy, &event);
       if (event.type == ClientMessage) {
-	if (event.xclient.message_type == gvpage) break;
+	 if (event.xclient.message_type == gvpage) break;
+         else if (event.xclient.message_type == gvdone) {
 #ifdef GS_DEBUG
-        else if (event.xclient.message_type == gvdone)
 	    printf("Xcircuit: Received DONE message from ghostscript\n");
 #endif
+	    /* Catastrophic error in ghostscript */
+	    return;
+         }
       } 
+      else if (event.type == KeyPress) {	/* Check for Ctrl-C break */
+	 XKeyEvent *keyevent = (XKeyEvent *)(&event);
+	 if (keyevent->state == 4 && keyevent->keycode == 54) {
+	    Wprintf("Ctrl-C break from render loop");
+	    break;
+	 }
+	 else {
+	    fprintf(stderr, "state = %d, keycode = %d\n", keyevent->state,
+		keyevent->keycode);
+	    XtDispatchEvent(&event);
+	 }
+      }
       else XtDispatchEvent(&event);	
    }
 #ifdef GS_DEBUG
@@ -397,7 +412,10 @@ void readbackground(FILE *fi)
 {
    FILE *fbg = (FILE *)NULL;
    int tfd;
-   char file_in[20] = "@/tmp/XXXXXX";  /* "@" denotes a temporary file */
+   char *file_in = (char *)malloc(9 + strlen(xobjs.tempdir));
+
+   /* "@" denotes a temporary file */
+   sprintf(file_in, "@%s/XXXXXX", xobjs.tempdir);
 
    tfd = mkstemp(file_in + 1);
    if (tfd == -1) printf("Error generating temporary filename\n");
@@ -416,6 +434,7 @@ void readbackground(FILE *fi)
       fclose(fbg);
       call_loadbg(file_in);
    }
+   free(file_in);
 }
 
 /*------------------------------------------------------*/
