@@ -117,7 +117,9 @@ void doxschema(Widget w, caddr_t clientdata, caddr_t calldata)
 
 void startconnect(Widget button, caddr_t clientdata, caddr_t calldata)
 {
-   if (eventmode == NORMAL_MODE) {
+   if (areastruct.selects > 0)
+      connectivity(button, clientdata, calldata);
+   else if (eventmode == NORMAL_MODE) {
       Wprintf("Click button1 on wire to see connectivity.");
       eventmode = CONNECT_MODE;
    }
@@ -130,10 +132,11 @@ void startconnect(Widget button, caddr_t clientdata, caddr_t calldata)
 
 void connectivity(Widget button, caddr_t clientdata, caddr_t calldata)
 {
+   short *gsel;
    genericptr ggen;
    int depth, netid;
    objectpair *seltop, *nextptr;
-   objectptr nettop;
+   objectptr nettop, thisobj;
    char *snew;
    stringpart *ppin;
 
@@ -142,17 +145,39 @@ void connectivity(Widget button, caddr_t clientdata, caddr_t calldata)
    seltop->thisinst = NULL;
    seltop->nextpair = NULL;
 
+   /* pick the first selection that looks like a valid network part */
+
+   if (areastruct.selects > 0) {
+      for (gsel = areastruct.selectlist; gsel < areastruct.selectlist +
+		areastruct.selects; gsel++) {
+	 ggen = *(objectdata->plist + *gsel);
+         if (SELECTTYPE(gsel) == LABEL) {
+	    labelptr glab = SELTOLABEL(gsel);
+	    if (glab->pin == LOCAL || glab->pin == GLOBAL) break;
+         }
+	 else if (SELECTTYPE(gsel) == POLYGON) {
+	    polyptr gpoly = SELTOPOLY(gsel);
+	    if (!nonnetwork(gpoly)) break;
+	 }
+      }
+      if (gsel == areastruct.selectlist + areastruct.selects) {
+	 Wprintf("Selected element is not part of a valid network.");
+      }
+   }
+   else
+      ggen = recurselect(&seltop);
+
    /* Determine the netid and the topmost object in which that	*/
    /* net appears.  Then build the transformation matrix down	*/
    /* to that object.  highlightnet() should end by popping the */
    /* entire matrix stack.					*/
 
-   if ((ggen = recurselect(&seltop)) != NULL) {
+   if (ggen != NULL) {
       createnets(True, NULL);
       if ((netid = is_resolved(&ggen, seltop, &nettop)) != 0) {
          /* printf("Net ID is %d in object %s\n", netid, nettop->name); */
          depth = pushnetwork(seltop, nettop);
-         /* printf(">> Pushed network %d levels deep\n", i); */
+         /* printf(">> Pushed network %d levels deep\n", depth); */
          nextptr = seltop;
          while (nextptr->thisobject != nettop)
 	    nextptr = nextptr->nextpair;
@@ -186,50 +211,6 @@ void connectivity(Widget button, caddr_t clientdata, caddr_t calldata)
    }
 }
 
-/*----------------------------------------------------------------------*/
-/* connectivity():  Find electrical connections into a node.		*/
-/* (simple search only on topmost object)				*/
-/*----------------------------------------------------------------------*/
-
-void tconnectivity(Widget button, caddr_t clientdata, caddr_t calldata)
-{
-   genericptr *ggen;
-   int netid;
-   short *csel;
-   objectpair *seltop;
-   objectptr retobj;
-
-   seltop = (objectpair *)malloc(sizeof(objectpair));
-   seltop->thisobject = objectdata;
-   seltop->thisinst = NULL;
-   seltop->nextpair = NULL;
-
-   objectselect(POLYGON | LABEL);
-   drawselects();
-   if (areastruct.selects == 0) {
-      Wprintf("Must select at least one polygon or label to find network connections.");
-   }
-   else {
-      createnets(True, NULL);
-      for (csel = areastruct.selectlist; csel < areastruct.selectlist +
-		areastruct.selects; csel++) {
-         ggen = objectdata->plist + (*csel);
-         netid = is_resolved(ggen, seltop, &retobj);
-         printf(">> Resolving connectivity for net %d\n", netid);
-	 if (retobj != objectdata)
-	    printf(">> Error:  Resolved object is not topmost object!\n");
-         if (netid != 0)
-            highlightnet(retobj, areastruct.topobject, netid);
-         else
-            Wprintf("No network is associated with this polygon");
-      }
-      destroynets();
-   }
-   clearselects();
-
-   free(seltop);
-}
-
 /*--------------------------------------------------------------*/
 /* Set object type to FUNDAMENTAL if it contains one or more	*/
 /* info labels for the given netlist mode.			*/
@@ -430,16 +411,23 @@ int findpageobj(objectptr pobj)
    return -1;
 }
 
-/*--------------------------------------------------------*/
-/* Recursively find all sub-circuits associated with the  */
-/* top-level circuit and set their filenames to be the	  */
-/* same.						  */
-/*--------------------------------------------------------*/
+/*-------------------------------------------------------*/
+/* Recursively find all sub-circuits associated with the */
+/* top-level circuit and set their filenames to be the	 */
+/* same.						 */
+/*							 */
+/* Avoid possible recursion problems by limiting the	 */
+/* number of recursion levels.  Presumably no circuit	 */
+/* would have more than several hundred hierarchical 	 */
+/* levels.						 */
+/*-------------------------------------------------------*/
 
-void findsubschems(int toppage, objectptr cschem)
+int findsubschems(int toppage, objectptr cschem, int level)
 {
    genericptr *cgen;
 
+   if (level == HIERARCHY_LIMIT) return -1;
+
    for (cgen = cschem->plist; cgen < cschem->plist + cschem->parts; cgen++) {
       if ((*cgen)->type == OBJECT) {
 	 objectptr cobj = TOOBJINST(cgen)->thisobject;
@@ -451,7 +439,10 @@ void findsubschems(int toppage, objectptr cschem)
 	       Wprintf("Error: disconnected schematic!\n");
 	    }
 	    else {
-	       xobjs.pagelist[pageno]->filename = xobjs.pagelist[toppage]->filename;
+	       if (xobjs.pagelist[pageno]->filename != NULL)
+		  free(xobjs.pagelist[pageno]->filename);
+	       xobjs.pagelist[pageno]->filename =
+			strdup(xobjs.pagelist[toppage]->filename);
 	    }
 
 	    /* A symbol on its own schematic page is allowed for clarity */
@@ -460,15 +451,18 @@ void findsubschems(int toppage, objectptr cschem)
 	    
 	    if (cobj->symschem != cschem) {
 	       /* printf("Searching for dependencies of symbol %s\n", cobj->name); */
-	       findsubschems(toppage, cobj->symschem);
+	       if (findsubschems(toppage, cobj->symschem, level + 1) == -1)
+		  return -1;
 	    }
 	 }
 	 else if (cobj->schemtype != FUNDAMENTAL && cobj->schemtype != TRIVIAL) {
 	    /* Check symbols acting as their own schematics */
-	    findsubschems(toppage, cobj);
+	    if (findsubschems(toppage, cobj, level + 1) == -1)
+	       return -1;
 	 }
       }
    }
+   return 0;
 }
 
 /*--------------------------------------------------------*/
@@ -507,6 +501,7 @@ void copypinlabel(labelptr pinlab)
       (*newlabel)->position.y = schemobj->lowerleft.y - TEXTHEIGHT
 	   * (*newlabel)->scale;
       schemobj->parts++;
+      incr_changes(schemobj);
       singlebbox(objectdata, ENDPART);
    }
 }
@@ -621,7 +616,7 @@ void swapschem(Widget w, u_int mode, caddr_t calldata)
    /* Create symbol or schematic, if allowed by mode */
 
    if ((objectdata->symschem == NULL) && (mode != 0)) {
-      if (objectdata->schemtype == SYMBOL) {
+      if (objectdata->schemtype != SCHEMATIC) {
 	 int tpage;
 
 	 /* create a new page for the new schematic */
@@ -694,6 +689,7 @@ void swapschem(Widget w, u_int mode, caddr_t calldata)
 	       u2u_snap(&((*pinlab)->position));
 	       (*pinlab)->string = stringcopy(lpin->string);
 	       objectdata->parts++;
+	       incr_changes(objectdata);
 	    }
          }
 	 calcbbox(objectdata);
