@@ -620,7 +620,7 @@ void selectarea()
       areastruct.origin.x = areastruct.save.x;
       areastruct.save.x = tmp;
    }
- 
+
    textpos = textend = 0;
    for (curgen = objectdata->plist; curgen < objectdata->plist
 	+ objectdata->parts; curgen++) {
@@ -842,8 +842,9 @@ genericptr recurselect(objectpair **seltop)
       clearselects();
    }
    else if ((selectobj = selectobject(OBJECT, selobj, selinst)) != NULL) {
-      objinstptr selinst = TOOBJINST(selobj->plist + (*selectobj));	
       objectpair *selnew = (objectpair *)malloc(sizeof(objectpair));
+
+      selinst = TOOBJINST(selobj->plist + (*selectobj));	
       drawselects(selobj, selinst);
       clearselects();
 
@@ -884,7 +885,8 @@ genericptr recurselect(objectpair **seltop)
 
 void startselect(XtIntervalId id, caddr_t clientdata)
 {
-   if (eventmode == NORMAL_MODE) eventmode = SELAREA_MODE;
+   if (eventmode != PENDING_MODE) return;
+   eventmode = SELAREA_MODE;
    areastruct.origin.x = areastruct.save.x;
    areastruct.origin.y = areastruct.save.y;
    UDrawBox(areastruct.origin, areastruct.save);
