@@ -4,6 +4,8 @@
 /*-----------------------------------------------------------------------*/
 
 #include <stdio.h>
+#include <stdlib.h>
+#include <ctype.h>   /* for isprint() */
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -76,10 +78,14 @@ stringpart *makesegment(stringpart **strhead, stringpart *before)
 	    if (lastptr->type == PARAM_START) {
 	       oparamptr obs = NULL;
 	       int paramno = lastptr->data.paramno;
-	       if (NORMINST->params != NULL)
-	          obs = NORMINST->params[paramno];
-	       if (obs == NULL)
-		  obs = NORMINST->thisobject->params[paramno];
+	       if (NORMINST == NULL)
+		  obs = objectdata->params[paramno];
+	       else {
+	          if (NORMINST->params != NULL)
+	             obs = NORMINST->params[paramno];
+	          if (obs == NULL)
+		     obs = NORMINST->thisobject->params[paramno];
+	       }
 	       if (obs == NULL) {
 		  Wprintf("Error:  Bad parameter");
 		  return NULL;
@@ -113,7 +119,7 @@ stringpart *splitstring(int tpos, stringpart **strtop, objinstptr localdata)
    stringpart *newpart, *ipart;
 
    ipart = findstringpart(tpos, &locpos, *strtop, localdata);
-   if (locpos != 0) {        /* split the string */
+   if (locpos > 0) {        /* split the string */
       newpart = makesegment(strtop, ipart);
       newpart->type = TEXT_STRING;
       newpart->data.string = ipart->data.string;
@@ -169,9 +175,16 @@ stringpart *deletestring(stringpart *dstr, stringpart **strtop, objinstptr thisi
       /* out if it's an instance or a default, and change the pointer	*/
       /* to the parameter in the parameter list, accordingly.		*/
 
-      else if ((strptr->type == PARAM_START) && (thisinst != NULL)) {
+      else if (strptr->type == PARAM_START) {
 	 paramno = strptr->data.paramno;
-	 if (thisinst->params == NULL || thisinst->params[paramno] == NULL)
+	 if (thisinst == NULL)
+	 {
+	    if (objectdata->params != NULL)
+	       objectdata->params[paramno]->parameter.string = dstr->nextpart;
+	    else
+	       fprintf(stderr, "Error in deletestring:  Bad parameter number found\n"); 
+	 }
+	 else if (thisinst->params == NULL || thisinst->params[paramno] == NULL)
 	    thisinst->thisobject->params[paramno]->parameter.string =
 			dstr->nextpart;
 	 else
@@ -263,6 +276,7 @@ stringpart *linkstring(objinstptr localdata, stringpart *strstart)
       tmpptr->nextpart = strstart->nextpart;
       return nextptr;
    }
+   return NULL;
 }
 
 /*----------------------------------------------------------------------*/
@@ -296,7 +310,7 @@ stringpart *findstringpart(int tpos, int *locpos, stringpart *strtop,
    int testpos = 0, tmplen;
 
    for (strptr = strtop; strptr != NULL; strptr = nextstringpart(strptr, localdata)) {
-      if (strptr->type == TEXT_STRING) {
+      if ((strptr->type == TEXT_STRING) && strptr->data.string) {
 	 tmplen = strlen(strptr->data.string);
 	 if (testpos + tmplen > tpos) {
 	    if (locpos != NULL) *locpos = (tpos - testpos);
@@ -337,14 +351,18 @@ void charprint(char *sout, stringpart *strptr, int locpos)
  
    switch (strptr->type) {
       case TEXT_STRING:
-         if (locpos > strlen(strptr->data.string)) {
-	    strcpy(sout, "<ERROR>");
+	 if (strptr->data.string) {
+            if (locpos > strlen(strptr->data.string)) {
+	       strcpy(sout, "<ERROR>");
+	    }
+            else sc = *(strptr->data.string + locpos);
+            if (isprint(sc))
+	       sprintf(sout, "%c", sc);
+            else
+	       sprintf(sout, "/%03o", (u_char)sc);
 	 }
-         else sc = *(strptr->data.string + locpos);
-         if (isprint(sc))
-	    sprintf(sout, "%c", sc);
-         else
-	    sprintf(sout, "/%03o", (u_char)sc);
+	 else
+	    *sout = '\0';
 	 break;
       case FONT_NAME:
 	 sprintf(sout, "Font=%s", fonts[strptr->data.font].psname);
@@ -401,7 +419,7 @@ char *xcstringtostring(stringpart *strtop, objinstptr localdata, Boolean textonl
 
 char *stringprint(stringpart *strtop, objinstptr localdata)
 {
-   xcstringtostring(strtop, localdata, False);
+   return xcstringtostring(strtop, localdata, False);
 }
 
 /*----------------------------------------------------------------------*/
@@ -411,7 +429,7 @@ char *stringprint(stringpart *strtop, objinstptr localdata)
 
 char *textprint(stringpart *strtop, objinstptr localdata)
 {
-   xcstringtostring(strtop, localdata, True);
+   return xcstringtostring(strtop, localdata, True);
 }
 
 /*----------------------------------------------------------------------*/
@@ -452,14 +470,14 @@ int textcompx(stringpart *string, char *text, Boolean exact, objinstptr localdat
 
 int textcomp(stringpart *string, char *text, objinstptr localdata)
 {
-   textcompx(string, text, True, localdata);
+   return textcompx(string, text, True, localdata);
 }
 
 /*----------------------------------------------------------------------*/
 
 int textncomp(stringpart *string, char *text, objinstptr localdata)
 {
-   textcompx(string, text, False, localdata);
+   return textcompx(string, text, False, localdata);
 }
 
 /*----------------------------------------------------------------------*/
@@ -477,7 +495,11 @@ int stringcomp(stringpart *string1, stringpart *string2)
       else {
          switch (strptr1->type) {
 	    case TEXT_STRING:
-	       if (strcmp(strptr1->data.string, strptr2->data.string))
+	       if (strptr1->data.string && strptr2->data.string) {
+	          if (strcmp(strptr1->data.string, strptr2->data.string))
+		     return 1;
+	       }
+	       else if (strptr1->data.string || strptr2->data.string)
 		  return 1;
 	       break;
 	    case FONT_SCALE:
@@ -512,6 +534,16 @@ Boolean issymbolfont(int fontnumber)
    return False;
 }
 
+/*----------------------------------------------------------------------*/
+/* Test if the specified font is ISO-Latin1 encoding			*/
+/*----------------------------------------------------------------------*/
+
+Boolean isisolatin1(int fontnumber)
+{
+   if ((fonts[fontnumber].flags & 0xf80) == 0x100) return True;
+   return False;
+}
+
 /*----------------------------------------------------------------------*/
 /* Test equivalence of two label strings (relaxed constraints)		*/
 /*    Like stringcomp(), but ignores "superficial" differences such as  */
@@ -590,8 +622,10 @@ int stringlength(stringpart *string, Boolean doparam, objinstptr thisinst)
 
    for (strptr = string; strptr != NULL; strptr = (doparam) ?
 		nextstringpart(strptr, thisinst) : strptr->nextpart) {
-      if (strptr->type == TEXT_STRING)
-	 ctotal += strlen(strptr->data.string);
+      if (strptr->type == TEXT_STRING) {
+	 if (strptr->data.string)
+	    ctotal += strlen(strptr->data.string);
+      }
       else
 	 ctotal++;
    }
@@ -605,10 +639,19 @@ int stringlength(stringpart *string, Boolean doparam, objinstptr thisinst)
 
 stringpart *stringcopy(stringpart *string)
 {
-   stringpart *strptr, *newpart, *newtop = NULL;
+   stringpart *strptr, *newpart, *newtop = NULL, *topptr;
 
    for (strptr = string; strptr != NULL; strptr = strptr->nextpart) {
-      newpart = makesegment(&newtop, NULL);
+
+      /* Don't use makesegment(), which looks at parameter contents */
+      newpart = (stringpart *)malloc(sizeof(stringpart));
+      newpart->nextpart = NULL;
+      if (newtop == NULL)
+	 newtop = newpart;
+      else 
+	 topptr->nextpart = newpart;
+      topptr = newpart;
+
       newpart->type = strptr->type;
       if (strptr->type == TEXT_STRING) {
 	 newpart->data.string = (char *)malloc(1 + strlen(strptr->data.string));
@@ -636,7 +679,7 @@ stringpart *stringcopyall(stringpart *string, objinstptr thisinst)
       if (strptr == string) newtop = newpart;
       else topend->nextpart = newpart;
       topend = newpart;
-      if (strptr->type == TEXT_STRING) {
+      if ((strptr->type == TEXT_STRING) && strptr->data.string) {
 	 newpart->data.string = (char *)malloc(1 + strlen(strptr->data.string));
 	 strcpy(newpart->data.string, strptr->data.string);
       }
@@ -653,7 +696,7 @@ stringpart *stringcopyall(stringpart *string, objinstptr thisinst)
 
 stringpart *stringcopyback(stringpart *string, objinstptr thisinst)
 {
-   stringpart *strptr, *newpart, *curend, *rettop, *curtop, *savend;
+   stringpart *strptr, *newpart, *curend = NULL, *rettop, *curtop, *savend;
    int paramno = -1;
    oparamptr pparam;
 
@@ -666,24 +709,38 @@ stringpart *stringcopyback(stringpart *string, objinstptr thisinst)
       if (strptr == string) rettop = newpart;	/* initial segment */
       else curend->nextpart = newpart;  	/* append segment to label */
 
-      if (curend->type == PARAM_START) {
-	 paramno = curend->data.paramno;
-	 curtop = newpart;
-	 savend = curend;
-      }
-      else if (curend->type == PARAM_END) {
-	 curend->nextpart = NULL;
-	 savend->nextpart = newpart;
+      if (curend) {
+         if (curend->type == PARAM_START) {
+	    paramno = curend->data.paramno;
+	    curtop = newpart;
+	    savend = curend;
+         }
+         else if (curend->type == PARAM_END) {
+	    curend->nextpart = NULL;
+	    savend->nextpart = newpart;
+         }
       }
       curend = newpart;
 
       if (strptr->type == TEXT_STRING) {
-	 newpart->data.string = (char *)malloc(1 + strlen(strptr->data.string));
-	 strcpy(newpart->data.string, strptr->data.string);
+	 if (strptr->data.string) {
+	    newpart->data.string = (char *)malloc(1 + strlen(strptr->data.string));
+	    strcpy(newpart->data.string, strptr->data.string);
+	 }
+	 else
+	    newpart->data.string = NULL;
       }
       else if (strptr->type == PARAM_END) {
+	 /* If thisinst is NULL, then use objectdata (top level object) */
+
 	 if (paramno >= 0) {
-	    if ((thisinst->params) != NULL && (thisinst->params[paramno] != NULL))
+	    if (thisinst == NULL) {
+	       if (objectdata->params != NULL)
+	          pparam = objectdata->params[paramno];
+	       else
+		  fprintf(stderr, "Error:  Bad parameter number encountered!\n");
+	    }
+	    else if ((thisinst->params) != NULL && (thisinst->params[paramno] != NULL))
 	       pparam = thisinst->params[paramno];
 	    else 
 	       pparam = thisinst->thisobject->params[paramno];
@@ -765,16 +822,16 @@ short UDrawChar(localdata, code, styles, ffont, groupheight, passcolor)
 
 void UDrawString(objinstptr localdata, labelptr drawlabel, int passcolor)
 {
-   stringpart *strptr, *tmpptr;
+   stringpart *strptr;
    char *textptr;
    short  fstyle, ffont, tmpjust, baseline;
-   int    paramno, pos, group = 0;
+   int    pos, group = 0;
    int    defaultcolor, curcolor, scolor;
    short  oldx, oldfont, oldstyle;
    float  tmpscale = 1.0, natscale = 1.0;
    float  tmpthick = xobjs.pagelist[areastruct.page]->wirewidth;
    XPoint newpoint, bboxin[2], bboxout[2];
-   Boolean xm, ym;
+   u_char xm, ym;
    TextExtents tmpext;
    short *tabstops = NULL;
    short tabno, numtabs = 0;
@@ -845,7 +902,7 @@ void UDrawString(objinstptr localdata, labelptr drawlabel, int passcolor)
    ym = (bboxout[0].y < bboxout[1].y) ? 0 : 1;
 
    if (bboxout[xm].x < areastruct.width && bboxout[ym].y < areastruct.height &&
-       bboxout[!xm].x > 0 && bboxout[!ym].y > 0) {
+       bboxout[1 - xm].x > 0 && bboxout[1 - ym].y > 0) {
 
        pos = 0;
        for (strptr = drawlabel->string; strptr != NULL;
@@ -959,9 +1016,9 @@ void UDrawString(objinstptr localdata, labelptr drawlabel, int passcolor)
 		   int tmpheight;
 
 		   group = 0;
-		   for (textptr = strptr->nextpart->data.string; *textptr != '\0';
+		   for (textptr = strptr->nextpart->data.string; textptr && *textptr != '\0';
 				textptr++) {
-		      charptr = fonts[ffont].encoding[*textptr];
+		      charptr = fonts[ffont].encoding[*(unsigned char *)textptr];
 		      tmpheight = (int)((float)charptr->height * fonts[ffont].scale);
 		      if (group < tmpheight) group = tmpheight;
 		   }
@@ -983,7 +1040,7 @@ void UDrawString(objinstptr localdata, labelptr drawlabel, int passcolor)
 		} break;
 
 	     case TEXT_STRING:
-		for (textptr = strptr->data.string; *textptr != '\0'; textptr++) {
+		for (textptr = strptr->data.string; textptr && *textptr != '\0'; textptr++) {
 		   pos++;
 		   UPushCTM();
 		   UPreMultCTM(DCTM, newpoint, tmpscale, 0);
@@ -1019,8 +1076,9 @@ enddraw:
 
    UPopCTM();
 
-   if ((defaultcolor != DOFORALL) && (passcolor != curcolor))
+   if ((defaultcolor != DOFORALL) && (passcolor != curcolor)) {
       XTopSetForeground(passcolor);
+   }
 
    xobjs.pagelist[areastruct.page]->wirewidth = tmpthick;
 }
@@ -1126,7 +1184,7 @@ TextExtents ULength(stringpart *string, objinstptr localdata, float newscale,
 	       natscale = locscale;
 	    break;
 	 case TEXT_STRING:
-	    for (textptr = strptr->data.string; *textptr != '\0'; textptr++) {
+	    for (textptr = strptr->data.string; textptr && *textptr != '\0'; textptr++) {
                if (dostop && (locpos >= dostop)) break;
 	       locpos++;
 
