@@ -228,7 +228,7 @@ static XtResource resources[] = {
   { XtNheight, XtCHeight, XtRInt, sizeof(int),
       XtOffset(ApplicationDataPtr, height), XtRString, "760"},
   { "timeout", "TimeOut", XtRInt, sizeof(int),
-      XtOffset(ApplicationDataPtr, timeout), XtRString, "15"}
+      XtOffset(ApplicationDataPtr, timeout), XtRString, "10"}
 };
 
 /*-------------------------------------------------------------------------*/
@@ -415,7 +415,7 @@ void createtoolbar (Widget abform, Widget aform)
 }
 
 /*----------------------------------------------------------------------*/
-/* Toolbar Resize (This is not working right).				*/
+/* Toolbar Resize							*/
 /*----------------------------------------------------------------------*/
 
 void resizetoolbar()
@@ -426,6 +426,7 @@ void resizetoolbar()
    Widget bwptr, lbwptr, bmax, lbmax;
    Dimension t_height, bheight, bwidth;
    Boolean is_top = True;
+   int pybuttons = 0;
 
    if (!XtIsRealized(toolbar)) return;
 
@@ -439,9 +440,18 @@ void resizetoolbar()
    XtnSetArg(XtNheight, &t_height);
    XtGetValues(XtParent(toolbar), wargs, n); n = 0;
 
+#ifdef HAVE_PYTHON
+   pybuttons = pytoolbuttons();
+#endif
+
    /* Realign the tool buttons inside the fixed space */
 
-   for (i = 0; i < toolbuttons; i++) {
+   for (i = 0; i < toolbuttons + pybuttons; i++) {
+#ifdef HAVE_PYTHON
+      if (i >= toolbuttons)
+	 bwptr = pytoolbutton(i - toolbuttons);
+      else
+#endif
       bwptr = XtNameToWidget(toolbar, ToolBar[i].name);
       if (bwptr == (Widget)NULL) break;
 
@@ -542,20 +552,11 @@ void createmenus (Widget form, Widget *firstbutton, Widget *lastbutton)
    *lastbutton = buttonw[i - 1];
 }
 
-/*--------------------------------------*/
-/* A gentle Ctrl-C shutdown		*/
-/*--------------------------------------*/
-
-void dointr(int signum)
-{
-   quit(NULL, NULL, NULL);
-}
-
 /*---------------*/
 /* Quit xcircuit */
 /*---------------*/
 
-void quit(Widget w, caddr_t clientdata, caddr_t calldata)
+void quit(Widget w, caddr_t nulldata)
 {
    int i;
    Matrixptr curmatrix, dmatrix;
@@ -612,6 +613,99 @@ void quit(Widget w, caddr_t clientdata, caddr_t calldata)
 
 }
 
+/*--------------------------------------------------------------*/
+/* Check to see if any objects in xcircuit have been modified	*/
+/* without saving.						*/
+/*--------------------------------------------------------------*/
+
+u_short countchanges(char **promptstr)
+{
+   int slen, i, j;
+   u_short changes = 0;
+   objectptr thisobj;
+
+   if (promptstr != NULL) slen = strlen(*promptstr);
+
+   for (i = 0; i < xobjs.pages; i++) {
+      if ((thisobj = xobjs.pagelist[i]->pageobj) != NULL) {
+         if (thisobj->changes > 0) {
+	    if (promptstr != NULL) {
+	       slen += strlen(thisobj->name) + 2;
+	       *promptstr = (char *)realloc(*promptstr, slen);
+	       if (changes > 0) strcat(*promptstr, ", ");
+	       strcat(*promptstr, thisobj->name);
+	    }
+	    changes += thisobj->changes;
+	 }
+      }
+   }
+
+   /* Check all library objects for unsaved changes */
+
+   for (i = 0; i < xobjs.numlibs; i++) {
+      for (j = 0; j < xobjs.userlibs[i].number; j++) {
+	 thisobj = *(xobjs.userlibs[i].library + j);
+	 if (thisobj->changes > 0) {
+	    if (promptstr != NULL) {
+	       slen += strlen(thisobj->name) + 2;
+	       *promptstr = (char *)realloc(*promptstr, slen);
+	       if (changes > 0) strcat(*promptstr, ", ");
+	       strcat(*promptstr, thisobj->name);
+	    }
+	    changes += thisobj->changes;
+	 }
+      }
+   }
+   return changes;
+}
+
+/*----------------------------------------------*/
+/* Check for conditions to approve program exit */
+/*----------------------------------------------*/
+
+void quitcheck(Widget w, caddr_t clientdata, caddr_t calldata)
+{
+   char *promptstr;
+   Boolean doprompt = False;
+   buttonsave *savebutton;
+
+   /* enable default interrupt signal handler during this time, so that */
+   /* a double Control-C will ALWAYS exit.				*/
+
+   signal(SIGINT, SIG_DFL);
+
+   promptstr = (char *)malloc(22);
+   strcpy(promptstr, "Unsaved changes in: ");
+
+   /* Check all page objects for unsaved changes */
+
+   doprompt = (countchanges(&promptstr) > 0) ? True : False;
+
+   /* If any changes have not been saved, generate a prompt */
+
+   if (doprompt) {
+      savebutton = (buttonsave *)malloc(sizeof(buttonsave));
+      getgeneric(savebutton, w, quitcheck, NULL);
+      promptstr = (char *)realloc(promptstr, strlen(promptstr) + 14);
+      strcat(promptstr, "\nQuit anyway?");
+      popupprompt(w, promptstr, NULL, quit, savebutton, False);
+      free(promptstr);
+   }
+   else {
+      free(promptstr);
+      quit(w, NULL);
+   }
+}
+
+/*--------------------------------------*/
+/* A gentle Ctrl-C shutdown		*/
+/*--------------------------------------*/
+
+void dointr(int signum)
+{
+   quitcheck(NULL, NULL, NULL);
+}
+
 /*--------------*/
 /* Null routine */
 /*--------------*/
@@ -668,24 +762,56 @@ void destroypopup(Widget button, popupstruct *callstruct, caddr_t calldata)
 
    free(callstruct->buttonptr);
    free(callstruct);
+
+   /* in the case of "quitcheck", we want to make sure that the signal	*/
+   /* handler is reset to default behavior if the quit command is	*/
+   /* canceled from inside the popup prompt window.			*/
+
+   signal(SIGINT, dointr);
 }
 
-/*-------------------------------------------------------------------------*/
-/* Pull text from the popup prompt buffer into a global string variable    */
-/*-------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* Toggle button showing number of pages of output.			*/
+/* Erase the filename and set everything accordingly.			*/
+/*----------------------------------------------------------------------*/
+
+void linkset(Widget button, propstruct *callstruct, caddr_t calldata)
+{
+   Arg wargs[1];
+
+   free(xobjs.pagelist[areastruct.page]->filename);
+   xobjs.pagelist[areastruct.page]->filename = (char *)malloc(1);
+   xobjs.pagelist[areastruct.page]->filename[0] = '\0';
+
+   XwTextClearBuffer(callstruct->textw);
+   getproptext(button, callstruct, calldata);
+
+   /* Change the select button back to "Apply" */
+   XtSetArg(wargs[0], XtNlabel, "Apply");
+   XtSetValues(callstruct->buttonw, wargs, 1);
+
+   /* Pop down the toggle button */
+   XtUnmanageChild(button);
+}
+
+/*----------------------------------------------------------------------*/
+/* Pull text from the popup prompt buffer into a global string variable */
+/*----------------------------------------------------------------------*/
 
 void xcgettext(Widget button, popupstruct *callstruct, caddr_t calldata)
 {
-   sprintf(_STR2, "%.249s", XwTextCopyBuffer(callstruct->textw)); 
+   if (callstruct->textw != NULL) {
+      sprintf(_STR2, "%.249s", XwTextCopyBuffer(callstruct->textw)); 
 
-   /* functions which use the file selector should look for directory name */
-   /* in string rather than a file name, and take appropriate action.	   */
+      /* functions which use the file selector should look for directory name */
+      /* in string rather than a file name, and take appropriate action.      */
 
-   if (callstruct->fileprompt == True) {
-      if (lookdirectory(_STR2)) {
-         newfilelist(XtNameToWidget(XtParent(button), "Filelist"),
-               callstruct->textw);
-	 return;
+      if (callstruct->fileprompt == True) {
+         if (lookdirectory(_STR2)) {
+            newfilelist(XtNameToWidget(XtParent(button), "Filelist"),
+                  callstruct->textw);
+	    return;
+         }
       }
    }
 
@@ -717,16 +843,17 @@ void getproptext(Widget button, propstruct *callstruct, caddr_t calldata)
    /* longer be pointing to the data.					*/
 
    Arg wargs[1];
-   short fileyes = (callstruct->setvalue == setfilename);
+   short file_yes = (callstruct->setvalue == setfilename);
 
    sprintf(_STR2, "%.249s", XwTextCopyBuffer(callstruct->textw)); 
    (*(callstruct->setvalue))(button, callstruct->dataptr);
 
    /* special stuff for filename changes */
 
-   if (fileyes) {
+   if (file_yes) {
       char blabel[50];
-      Widget wrbutton;
+      short num_linked;
+      Widget wrbutton, ltoggle;
       struct stat statbuf;
 
       /* get updated file information */
@@ -751,6 +878,15 @@ void getproptext(Widget button, propstruct *callstruct, caddr_t calldata)
       wrbutton = XtNameToWidget(XtParent(button), "Write File");
       XtSetArg(wargs[0], XtNlabel, blabel);
       XtSetValues(wrbutton, wargs, 1);
+
+      num_linked = pagelinks();
+      if (num_linked > 1) {
+	 ltoggle = XtNameToWidget(XtParent(button), "LToggle");
+	 sprintf(blabel, "%d Pages", num_linked);
+	 XtSetArg(wargs[0], XtNlabel, blabel);
+	 XtSetValues(ltoggle, wargs, 1);
+	 XtManageChild(ltoggle);
+      }
    }
 
    /* objectdata->name is not malloc'd, so is not changed by call to */
@@ -767,9 +903,9 @@ void getproptext(Widget button, propstruct *callstruct, caddr_t calldata)
    XtSetValues(callstruct->buttonw, wargs, 1);
 }
 
-/*-------------------------------------------------------------------------*/
-/* Update scale, width, and height in response to change of one of them	   */
-/*-------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* Update scale, width, and height in response to change of one of them	*/
+/*----------------------------------------------------------------------*/
 
 void updatetext(Widget button, WidgetList callstruct, caddr_t calldata)
 {
@@ -892,18 +1028,22 @@ void popupprompt(Widget button, char *request, char *current, void (*function)()
     okaystruct->buttonptr = datastruct;
     okaystruct->popup = popup;
     okaystruct->fileprompt = fileprompt;
+    okaystruct->setvalue = function;
+    okaystruct->textw = NULL;
+
+    XtnSetArg(XtNx, width - bwidth - 20);
+    XtnSetArg(XtNy, height - ROWHEIGHT - 10);
+    XtnSetArg(XtNwidth, bwidth);
+    XtnSetArg(XtNfont, appdata.xcfont);
+    cancelbutton = XtCreateManagedWidget("Cancel", XwmenuButtonWidgetClass, 
+	   dialog, wargs, n); n = 0; 
+
+    XtAddCallback(cancelbutton, XtNselect, (XtCallbackProc)destroypopup, okaystruct);
 
     if (current != NULL) {   /* A Text Edit widget is required */
        char		*pdptr;
        short		posit;
 
-       XtnSetArg(XtNx, width - bwidth - 20);
-       XtnSetArg(XtNy, height - ROWHEIGHT - 10);
-       XtnSetArg(XtNwidth, bwidth);
-       XtnSetArg(XtNfont, appdata.xcfont);
-       cancelbutton = XtCreateManagedWidget("Cancel", XwmenuButtonWidgetClass, 
-	   dialog, wargs, n); n = 0; 
-
        XtnSetArg(XtNx, 20);
        XtnSetArg(XtNy, ROWHEIGHT + 10 + (fileprompt ? LISTHEIGHT : 0));
        XtnSetArg(XtNheight, ROWHEIGHT + 5);
@@ -919,9 +1059,7 @@ void popupprompt(Widget button, char *request, char *current, void (*function)()
 	   dialog, wargs, n); n = 0; 
 
        okaystruct->textw = entertext;
-       okaystruct->setvalue = function;
 
-       XtAddCallback(okbutton, XtNselect, (XtCallbackProc)xcgettext, okaystruct);
        XtAddEventHandler(dialog, KeyPressMask, False,
 	  (XtEventHandler)propevent, entertext);
        XtAddEventHandler(staticarea, KeyPressMask, False,
@@ -929,13 +1067,12 @@ void popupprompt(Widget button, char *request, char *current, void (*function)()
        XtOverrideTranslations(entertext, XtParseTranslationTable
 	  (defaultTranslations));
        XtAddCallback(entertext, XtNexecute, (XtCallbackProc)xcgettext, okaystruct);
-       XtAddCallback(cancelbutton, XtNselect, (XtCallbackProc)destroypopup, okaystruct);
 
        /* Generate file prompting widget */
 
        if (fileprompt) genfilelist(dialog, entertext, width);
     }
-    else XtAddCallback(okbutton, XtNselect, (XtCallbackProc)destroypopup, okaystruct);
+    XtAddCallback(okbutton, XtNselect, (XtCallbackProc)xcgettext, okaystruct);
 
     XtPopup(popup, XtGrabNone);
 
@@ -957,6 +1094,8 @@ void popupprompt(Widget button, char *request, char *current, void (*function)()
 /*-------------------------------------------------------------------------*/
 
 #define MAXPROPS 7
+#define MARGIN 15
+
 propstruct okstruct[MAXPROPS], fpokstruct;
 
 void dooutput(Widget button, caddr_t clientdata, caddr_t calldata)
@@ -964,9 +1103,10 @@ void dooutput(Widget button, caddr_t clientdata, caddr_t calldata)
    buttonsave  *savebutton;
    Arg         wargs[9];
    Widget      popup, dialog, okbutton, titlearea, wrbutton;
-   Widget      fpentertext, fpokay, autobutton;
+   Widget      fpentertext, fpokay, autobutton, allpages;
    WidgetList  staticarea, entertext, okays;
    XWMHints    *wmhints;	/* for proper input focus */
+   short       num_linked;
    Position    xpos, ypos;
    short       n = 0;
    Dimension   height, width, areawidth, areaheight, bwidth, owidth, wwidth;
@@ -976,7 +1116,7 @@ void dooutput(Widget button, caddr_t clientdata, caddr_t calldata)
    void		(*function[MAXPROPS])();
    void		(*update[MAXPROPS])();
    char	statics[MAXPROPS][50], edit[MAXPROPS][75], request[150];
-   char fpedit[75], outname[75];
+   char fpedit[75], outname[75], pstr[20];
    void	*data[MAXPROPS];
    float oscale, psscale;
    struct stat statbuf;
@@ -1123,7 +1263,7 @@ void dooutput(Widget button, caddr_t clientdata, caddr_t calldata)
 
    for (i = 0; i < MAXPROPS; i++) {
       XtnSetArg(XtNx, 20);
-      XtnSetArg(XtNy, ROWHEIGHT + 15 + (i * 2 * ROWHEIGHT));
+      XtnSetArg(XtNy, ROWHEIGHT + MARGIN + 5 + (i * 2 * ROWHEIGHT));
       XtnSetArg(XtNstring, statics[i]);
       XtnSetArg(XtNborderWidth, 0);
       XtnSetArg(XtNgravity, WestGravity);
@@ -1132,7 +1272,7 @@ void dooutput(Widget button, caddr_t clientdata, caddr_t calldata)
 	   dialog, wargs, n); n = 0;
 
       XtnSetArg(XtNx, 150);
-      XtnSetArg(XtNy, ROWHEIGHT + 10 + (i * 2 * ROWHEIGHT));
+      XtnSetArg(XtNy, ROWHEIGHT + MARGIN + (i * 2 * ROWHEIGHT));
       if (i < 5) {
          XtnSetArg(XtNheight, ROWHEIGHT + 5);
          XtnSetArg(XtNstring, edit[i]);
@@ -1147,11 +1287,11 @@ void dooutput(Widget button, caddr_t clientdata, caddr_t calldata)
 	     dialog, wargs, n); n = 0; 
 
          XtnSetArg(XtNx, width - owidth - 20);
-         XtnSetArg(XtNy, ROWHEIGHT + 10 + (i * 2 * ROWHEIGHT));
+         XtnSetArg(XtNy, ROWHEIGHT + MARGIN + (i * 2 * ROWHEIGHT));
          XtnSetArg(XtNwidth, owidth); 
          XtnSetArg(XtNfont, appdata.xcfont);
-         okays[i] = XtCreateManagedWidget("Apply", XwmenuButtonWidgetClass, 
-	    dialog, wargs, n); n = 0;
+         okays[i] = XtCreateManagedWidget("Apply", XwmenuButtonWidgetClass,
+			dialog, wargs, n); n = 0;
 
          okstruct[i].textw = entertext[i];
 	 okstruct[i].buttonw = okays[i];
@@ -1180,12 +1320,27 @@ void dooutput(Widget button, caddr_t clientdata, caddr_t calldata)
       }
    }
 
+   /* If this filename is linked to other pages (multi-page output), add a button */
+   /* which will unlink the page name from the other pages when toggled.	  */
+
+   num_linked = pagelinks();
+   XtnSetArg(XtNx, width - wwidth - 20);
+   XtnSetArg(XtNy, ROWHEIGHT - 10);
+   XtnSetArg(XtNset, True);
+   XtnSetArg(XtNsquare, True);
+   XtnSetArg(XtNborderWidth, 0);
+   XtnSetArg(XtNfont, appdata.xcfont);
+   sprintf(pstr, "%d Pages", num_linked);
+   XtnSetArg(XtNlabel, pstr);
+   allpages = XtCreateWidget("LToggle", XwtoggleWidgetClass, dialog, wargs, n); n = 0;
+   XtAddCallback(allpages, XtNrelease, (XtCallbackProc)linkset, &okstruct[0]);
+
    /* If full-page pmode is chosen, there is an additional text structure.
       Make this text structure always but allow it to be managed and
       unmanaged as necessary. */
 
    XtnSetArg(XtNx, 240);
-   XtnSetArg(XtNy, ROWHEIGHT + 10 + (10 * ROWHEIGHT));
+   XtnSetArg(XtNy, ROWHEIGHT + MARGIN + (10 * ROWHEIGHT));
    XtnSetArg(XtNset, (xobjs.pagelist[areastruct.page]->pmode & 2) ? True : False);
    XtnSetArg(XtNsquare, True);
    XtnSetArg(XtNborderWidth, 0);
@@ -1204,7 +1359,7 @@ void dooutput(Widget button, caddr_t clientdata, caddr_t calldata)
 	 (float)xobjs.pagelist[areastruct.page]->pagesize.y / 72.0);
    }
    XtnSetArg(XtNx, 240);
-   XtnSetArg(XtNy, ROWHEIGHT + 10 + (12 * ROWHEIGHT));
+   XtnSetArg(XtNy, ROWHEIGHT + MARGIN + (12 * ROWHEIGHT));
    XtnSetArg(XtNheight, ROWHEIGHT + 5);
    XtnSetArg(XtNstring, fpedit);
    XtnSetArg(XtNwidth, width - owidth - 280);
@@ -1218,7 +1373,7 @@ void dooutput(Widget button, caddr_t clientdata, caddr_t calldata)
        dialog, wargs, n); n = 0;
 
    XtnSetArg(XtNx, width - owidth - 20);
-   XtnSetArg(XtNy, ROWHEIGHT + 10 + (12 * ROWHEIGHT));
+   XtnSetArg(XtNy, ROWHEIGHT + MARGIN + (12 * ROWHEIGHT));
    XtnSetArg(XtNwidth, owidth);
    XtnSetArg(XtNfont, appdata.xcfont);
    XtnSetArg(XtNlabel, "Apply");
@@ -1245,6 +1400,10 @@ void dooutput(Widget button, caddr_t clientdata, caddr_t calldata)
       XtManageChild(autobutton);
    }
 
+   if (num_linked > 1) {
+      XtManageChild(allpages);
+   }
+
    /* end of pagesize extra Widget definitions */
 
    donestruct = (popupstruct *) malloc(sizeof(popupstruct));
@@ -1340,6 +1499,134 @@ void Wprintf(char *string)
    printtime_id = XtAddTimeOut(10000, (XtTimerCallbackProc)clrmessage, NULL);
 }   
 
+/*------------------------------------------------------------------------------*/
+/* diagnostic tool for translating the event mode into a string and printing	*/
+/* to stderr (for debugging only).						*/
+/*------------------------------------------------------------------------------*/
+
+void printeventmode() {
+   fprintf(stderr, "eventmode is \'");
+   switch(eventmode) {
+      case NORMAL_MODE:
+	 fprintf(stderr, "NORMAL");
+	 break;
+      case PENDING_MODE:
+	 fprintf(stderr, "PENDING");
+	 break;
+      case PRESS_MODE:
+	 fprintf(stderr, "PRESS");
+	 break;
+      case DELETE_MODE:
+	 fprintf(stderr, "DELETE");
+	 break;
+      case COPY_MODE:
+	 fprintf(stderr, "COPY");
+	 break;
+      case COPY2_MODE:
+	 fprintf(stderr, "COPY2");
+	 break;
+      case ROTATE_MODE:
+	 fprintf(stderr, "ROTATE");
+	 break;
+      case PUSH_MODE:
+	 fprintf(stderr, "PUSH");
+	 break;
+      case EDIT_MODE:
+	 fprintf(stderr, "EDIT");
+	 break;
+      case SELAREA_MODE:
+	 fprintf(stderr, "SELAREA");
+	 break;
+      case SELAREA2_MODE:
+	 fprintf(stderr, "SELAREA2");
+	 break;
+      case DESEL_MODE:
+	 fprintf(stderr, "DESEL");
+	 break;
+      case PAN_MODE:
+	 fprintf(stderr, "PAN");
+	 break;
+      case CATPAN_MODE:
+	 fprintf(stderr, "CATPAN");
+	 break;
+      case CATALOG_MODE:
+	 fprintf(stderr, "CATALOG");
+	 break;
+      case CATTEXT_MODE:
+	 fprintf(stderr, "CATTEXT");
+	 break;
+      case FONTCAT_MODE:
+	 fprintf(stderr, "FONTCAT");
+	 break;
+      case FONTCAT2_MODE:
+	 fprintf(stderr, "FONTCAT2");
+	 break;
+      case TEXT1_MODE:
+	 fprintf(stderr, "TEXT1");
+	 break;
+      case TEXT2_MODE:
+	 fprintf(stderr, "TEXT2");
+	 break;
+      case TEXT3_MODE:
+	 fprintf(stderr, "TEXT3");
+	 break;
+      case WIRE_MODE:
+	 fprintf(stderr, "WIRE");
+	 break;
+      case BOX0_MODE:
+	 fprintf(stderr, "BOX0");
+	 break;
+      case BOX_MODE:
+	 fprintf(stderr, "BOX");
+	 break;
+      case EBOX_MODE:
+	 fprintf(stderr, "EBOX");
+	 break;
+      case ARC0_MODE:
+	 fprintf(stderr, "ARC0");
+	 break;
+      case ARC_MODE:
+	 fprintf(stderr, "ARC");
+	 break;
+      case EARC_MODE:
+	 fprintf(stderr, "EARC");
+	 break;
+      case SPLINE0_MODE:
+	 fprintf(stderr, "SPLINE0");
+	 break;
+      case SPLINE_MODE:
+	 fprintf(stderr, "SPLINE");
+	 break;
+      case ESPLINE_MODE:
+	 fprintf(stderr, "ESPLINE");
+	 break;
+      case EPATH_MODE:
+	 fprintf(stderr, "EPATH");
+	 break;
+      case LPARAM_MODE:
+	 fprintf(stderr, "LPARAM");
+	 break;
+      case IPARAM_MODE:
+	 fprintf(stderr, "IPARAM");
+	 break;
+      case ULPARAM_MODE:
+	 fprintf(stderr, "ULPARAM");
+	 break;
+      case UIPARAM_MODE:
+	 fprintf(stderr, "UIPARAM");
+	 break;
+#ifdef SCHEMA
+      case CONNECT_MODE:
+	 fprintf(stderr, "CONNECT");
+	 break;
+      case ASSOC_MODE:
+	 fprintf(stderr, "ASSOC");
+	 break;
+#endif
+   }
+   fprintf(stderr, "_MODE\'\n");
+}
+
 /*------------------------------------------------------------------------------*/
 
 void getcommand(Widget cmdw, caddr_t clientdata, caddr_t calldata)
@@ -1353,6 +1640,7 @@ void getcommand(Widget cmdw, caddr_t clientdata, caddr_t calldata)
    XtRemoveEventHandler(areastruct.area, KeyPressMask, False,
 	(XtEventHandler)propevent, cmdw);
    XtAddCallback(areastruct.area, XtNkeyDown, (XtCallbackProc)keyhandler, NULL);
+   XtAddCallback(areastruct.area, XtNkeyUp, (XtCallbackProc)keyhandler, NULL);
    XtUnmanageChild(cmdw);
 }
 
@@ -1400,6 +1688,7 @@ void docommand()
    /* temporarily redirect all text into the overlay widget */
 
    XtRemoveCallback(areastruct.area, XtNkeyDown, (XtCallbackProc)keyhandler, NULL);
+   XtRemoveCallback(areastruct.area, XtNkeyUp, (XtCallbackProc)keyhandler, NULL);
    XtAddEventHandler(areastruct.area, KeyPressMask, False,
 	(XtEventHandler)propevent, overlay);
 }
@@ -1759,9 +2048,16 @@ int main(int argc, char *argv[])
    xobjs.pagelist[0]->pagesize.x = 612;
    xobjs.pagelist[0]->pagesize.y = 792;
 
+   xobjs.new_changes = 0;
    xobjs.tempfile = NULL;
    signal(SIGINT, dointr);
 
+   /* Set the temporary directory name as compiled, unless overridden by */
+   /* environment variable "TMPDIR".					 */
+
+   xobjs.tempdir = getenv("TMPDIR");
+   if (xobjs.tempdir == NULL) xobjs.tempdir = strdup(TEMP_DIR);
+
    areastruct.area = (Widget)NULL;
    areastruct.mapped = False;
 
@@ -1771,6 +2067,7 @@ int main(int argc, char *argv[])
 
    areastruct.textscale = 1.0;
    areastruct.linewidth = 1.0;
+   areastruct.zoomfactor = SCALEFAC;
    areastruct.style = UNCLOSED;
    areastruct.invert = False;
    areastruct.axeson = True;
@@ -1826,14 +2123,15 @@ int main(int argc, char *argv[])
 
    XtSetLanguageProc(NULL, NULL, NULL);
 
-   /*-----------------------------------*/
-   /* Set pointer to own XDefaults file */
-   /*-----------------------------------*/
+   /*-------------------------------------------------------------------*/
+   /* Set pointer to own XDefaults file, but allow an external override */
+   /*-------------------------------------------------------------------*/
 
 #ifdef HAVE_PUTENV
-   putenv("XAPPLRESDIR=" XAPPLRESDIR);
+   if (getenv("XAPPLRESDIR") == NULL)
+      putenv("XAPPLRESDIR=" RESOURCES_DIR);
 #else
-   setenv("XAPPLRESDIR", XAPPLRESDIR, 1);
+   setenv("XAPPLRESDIR", RESOURCES_DIR, 0);
 #endif
 
    /*-----------------------------*/
@@ -2071,9 +2369,10 @@ int main(int argc, char *argv[])
    XtAddCallback(areastruct.area, XtNexpose, (XtCallbackProc)drawarea, NULL);
    XtAddCallback(areastruct.area, XtNresize, (XtCallbackProc)resizearea, NULL);
 
-   XtAddCallback(areastruct.area, XtNselect, (XtCallbackProc)selectbutton, NULL);
-   XtAddCallback(areastruct.area, XtNrelease, (XtCallbackProc)releasebutton, NULL);
+   XtAddCallback(areastruct.area, XtNselect, (XtCallbackProc)buttonhandler, NULL);
+   XtAddCallback(areastruct.area, XtNrelease, (XtCallbackProc)buttonhandler, NULL);
    XtAddCallback(areastruct.area, XtNkeyDown, (XtCallbackProc)keyhandler, NULL);
+   XtAddCallback(areastruct.area, XtNkeyUp, (XtCallbackProc)keyhandler, NULL);
 
    /* Setup callback routines for the scrollbar widgets */
 
@@ -2233,9 +2532,7 @@ int main(int argc, char *argv[])
 
    objectdata = xobjs.pagelist[0]->pageobj;
    sprintf(objectdata->name, "Page 1");
-   xobjs.pagelist[0]->filename = (char *)malloc((strlen(objectdata->name) + 1)
-	* sizeof(char));
-   strcpy(xobjs.pagelist[0]->filename, objectdata->name);
+   xobjs.pagelist[0]->filename = strdup(objectdata->name);
    initmem(objectdata);
    setpage();
 
