@@ -129,6 +129,7 @@ typedef struct {
 #define SPLINESEGS 20 /* Number of points per spline approximation 	*/
 #define DELBUFSIZE 10 /* Number of delete events to save for undeleting */
 #define MINAUTOSCALE 0.75 /* Won't automatically scale closer than this */
+#define MAXCHANGES 20 /* Number of changes to induce a temp file save	*/
 
 #ifdef SCHEMA
 #define PADSPACE   20 /* Spacing of pinlabels from their origins	*/
@@ -169,7 +170,7 @@ typedef struct {
 /*----------------------------------------------------------------------*/
 
 enum editmode {NORMAL_MODE = 0, 
-PRESS_MODE, DELETE_MODE, COPY_MODE, COPY2_MODE, ROTATE_MODE,
+PENDING_MODE, PRESS_MODE, DELETE_MODE, COPY_MODE, COPY2_MODE, ROTATE_MODE,
 PUSH_MODE, EDIT_MODE, SELAREA_MODE, SELAREA2_MODE, DESEL_MODE, PAN_MODE,
 CATPAN_MODE, CATALOG_MODE, CATTEXT_MODE, FONTCAT_MODE, FONTCAT2_MODE,
 TEXT1_MODE, TEXT2_MODE, TEXT3_MODE, WIRE_MODE, BOX0_MODE, BOX_MODE,
@@ -317,6 +318,8 @@ UIPARAM_MODE
 #define GLOBAL 2
 #define INFO 3
 
+#define HIERARCHY_LIMIT 256	/* Stop if recursion goes this deep */
+
 #endif
 
 /*----------------------------------------------------------------------*/
@@ -601,6 +604,7 @@ struct Ptab {
 
 typedef struct _xcobject {
    char		name[80];
+   u_short	changes;	/* Number of unsaved changes to object */
    Boolean	hidden;
    float	viewscale;
    XPoint	pcorner;	/* position relative to window */
@@ -791,7 +795,8 @@ enum {
    XCF_Pop, XCF_Push, XCF_Delete, XCF_Select, XCF_Box, XCF_Arc, XCF_Text,
    XCF_Exchange, XCF_Copy, XCF_Join, XCF_Unjoin, XCF_Spline, XCF_Edit,
    XCF_Undelete, XCF_Select_Save, XCF_Unselect, XCF_Dashed, XCF_Dotted,
-   XCF_Solid, XCF_Prompt, XCF_Dot, XCF_Wire, XCF_Nothing, XCF_Exit,
+   XCF_Solid, XCF_Prompt, XCF_Dot, XCF_Wire, XCF_Start, XCF_Finish,
+   XCF_Cancel, XCF_Nothing, XCF_Exit,
 
 #ifdef SCHEMA
    XCF_Netlist, XCF_Swap, XCF_Pin_Label, XCF_Info_Label, XCF_Connectivity,
@@ -843,6 +848,7 @@ typedef struct {
    /* global option defaults */
    float	textscale;
    float	linewidth;
+   float	zoomfactor;
    short	psfont;
    short	justify;
    short	style;
@@ -889,8 +895,10 @@ typedef struct {
 
 typedef struct {
    char		*tempfile;
+   char		*tempdir;
    int		timeout_id;
    int		save_interval;
+   u_short	new_changes;
    short	pages;
    short	numlibs;
    Pagedata	**pagelist;
